#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using NinjaTrader;
using System.Reflection;
using System.Windows.Forms;
#endregion


namespace NinjaTrader.Indicator {
	


public abstract class BaseFibLine 
	{
	protected Color color;
	protected ChartControl chart;
    protected IndicatorBase indicator;
	public FibonacciCircle FibonacciCircle;
	
	protected int position = 1;
	protected double level = 38.2;
	protected bool extended = true;
   
	
	public BaseFibLine (IndicatorBase indicator,FibonacciCircle circle, double level, int position, Color color,bool extended) {
		this.chart=indicator.ChartControl;
		this.color = color;
		this.indicator = indicator;
		this.extended = extended;
		FibonacciCircle = circle;
		
		if (position==1 || position==2 || position==3 || position ==4) {
			this.position = position;
		} else {
			throw new Exception("Invalid position argument  - should be 1, 2, 3, or 4");
		}
		if (level>0 && level<161.8) {
			this.level = level;
		} else {
			throw new Exception("Invalid extension, should be > 0 or < 161.8");
		}	
	}	
	
		public static int[] get_horizontal_line (IndicatorBase indicator, FibonacciCircle circle, bool is_top) {
			Point start  = getStart(circle,indicator);
			Point end = getEnd(circle,indicator);
			
		    double x_start = start.X;
			double y_start = start.Y;
			double x_end = end.X;
			double y_end = end.Y;
						
			double deltax = x_end-x_start;
			double deltay = y_end-y_start;
			
			double radius = Math.Sqrt(deltax*deltax+deltay*deltay);
			double outer_radius = radius*1.618;
	
			int [] positions  = new int[2];
			positions [0] = (int)(y_start- outer_radius);
			positions [1] = (int)( y_start + outer_radius);	
			return positions;
		}
	
	private static int getXByBarIdxDouble(double barDouble, ChartControl chart) {
			return (int)typeof(ChartControl).GetMethod("GetXByBarIdxDouble",BindingFlags.NonPublic|BindingFlags.Instance).Invoke(chart,new Object [] {barDouble});
	
	}
	
	public static Point getEnd(FibonacciCircle circle, IndicatorBase  indicator) {
		ChartControl chart = indicator.ChartControl;
			if (circle==null) return new Point(0,0);
			int x,y;
			if (chart.EquidistantBars==true) {
				double end_bar_double = (double)typeof(FibonacciRetracements).GetProperty("endBarDouble",BindingFlags.NonPublic | BindingFlags.Instance).GetValue(circle,null);
				x = getXByBarIdxDouble(end_bar_double,chart);
				y = chart.GetYByValue(indicator,circle.EndY);
				return new Point(x,y);
			}
			x = chart.GetXByTime(circle.EndTime);
		    y = chart.GetYByValue(indicator,circle.EndY);
			return new Point(x,y);
		}
		
	public static Point getStart (FibonacciCircle circle,IndicatorBase indicator) {
		ChartControl chart = indicator.ChartControl;
		if (circle==null) return new Point(0,0);
		int x,y;
		if (chart.EquidistantBars==true) {
			double start_bar_double = (double)typeof(ChartObject).GetProperty("startBarDouble",BindingFlags.NonPublic | BindingFlags.Instance).GetValue(circle,null);
			x = getXByBarIdxDouble(start_bar_double,chart);
			y = chart.GetYByValue(indicator,circle.StartY);
			return new Point(x,y);
		}
		
		x = chart.GetXByTime(circle.StartTime);
		y=  chart.GetYByValue(indicator,circle.StartY);
		return new Point(x,y);
	}
	
	
	
	/*
	protected double getPriceByY (double y,double min, double max) {
		int  price_CanvasBottom = chart.GetYByValue(indicator,min);
		int price_CanvasTop = chart.GetYByValue(indicator,max);

		return (max-(max-min)/(price_CanvasBottom-price_CanvasTop)*(y-price_CanvasTop));
		
	}

	public DateTime getTimeByX (int x) {
		MethodInfo info = chart.GetType().GetMethod("GetTimeByX",BindingFlags.NonPublic| BindingFlags.Instance);
		DateTime time = (DateTime)(info.Invoke(chart,new object[] {(int)x}));
		return time;
	}
	*/
	

		
		private static bool inBounds(Rectangle rect, double x, double y) {
			if (x>=rect.Left && x<=rect.Right && y>=rect.Top && y<=rect.Bottom) {
				return true;
			}
			return false;
		}
		
		public static Point[] calculateRectangleLineIntersection (Rectangle rect, double x1, double y1, double m) {
			double x2 = x1 + 4;
			double y2 = y1 + m*(x2-x1);
			return calculateRectangleLineIntersection(rect,x1,y1,x2,y2);		
		}
		
		public static Point[] calculateRectangleLineIntersection (Rectangle rect, double x1, double y1, double x2, double y2) {
			double m = (y2-y1)/(x2-x1);
			Point[] result = new Point[2];
			int index = 0;
			double  x = 0, y =0;
			//calculate possible 4 points of intersection and determine if it is in the rectangle
			//y-y1 = m*(x-x1)
			
			x = rect.Left; y = m*(x-x1)+y1; 
			if (inBounds(rect,x,y)) {
				result[index] = new Point((int)x,(int)y);
				++index;
			}
			
			x = rect.Right; y = m*(x-x1)+y1;
			if (inBounds(rect,x,y)) {
				result[index] = new Point((int)x,(int)y);
				++index;
			}
			
			if (index==2) return result;
			y = rect.Bottom; x = (y-y1)/m+x1;
			if (inBounds(rect,x,y)) {
				result[index] = new Point((int)x,(int)y);
				++index;
			}
			
			if (index==2) return result;
			y = rect.Top; x = (y-y1)/m+x1;
			if (inBounds(rect,x,y)) {
				result[index] = new Point((int)x,(int)y);
				++index; 
			}		
			
			if (index==2) return result;
			return null;

		}
		
	public abstract void draw(Graphics g, Rectangle bounds, double min, double max);
	public abstract void delete();

}

public class FibTimeLine : BaseFibLine
	{		
		public FibTimeLine (FibSR indicator,FibonacciCircle circle, double level, int position, Color color,bool extended):base(indicator,circle,level,position,color,extended) {}
		
		public override void delete () {}
		
		
		public static double[] get_tangent_line(IndicatorBase indicator, FibonacciCircle circle, double ratio, int pos)  {
			Point start  = getStart(circle,indicator);
			Point end = getEnd(circle,indicator);
			
		    double x_start = start.X;
			double y_start = start.Y;
			double x_end = end.X;
			double y_end = end.Y;
						
			double deltax = x_end-x_start;
			double deltay = y_end-y_start;
			
			double radius = Math.Sqrt(deltax*deltax+deltay*deltay);
			double outer_radius = radius*1.618;
			radius = ratio/100.0*radius; 
		
			//circle center coordinates
			double x = x_start;
			double y = -1*y_start;
			
			//top point coordinates
			double x1=0,y1=0;
			if (pos==1 || pos ==2) {
				x1 = x - outer_radius;
				y1 = y;
			} else {
				x1 = x + outer_radius;
				y1 = y;
			}
			
			//tangent point coordinates
			double x2 = radius*radius/(x1-x)+x;
			double y2 = -1*radius/(x1-x)*Math.Sqrt((x1-x)*(x1-x)-radius*radius);
	
			if (pos==2 || pos==3) {
				y2 = -y2;
			}
			
			y2 = y2 + y;	
			double [] coords = {x2,y2,x1,y1};
			return coords;
		}
		
		
		public override void draw (Graphics g, Rectangle bounds, double min, double max) 
		{
			double [] coordinates = FibTimeLine.get_tangent_line(indicator,FibonacciCircle,this.level,this.position);
			double x2 = coordinates[0]; double y2 = coordinates[1];
			double x1 = coordinates[2]; double y1 = coordinates[3];
			
			//convert to drawing coordinates
			y1 = -y1; y2 = -y2;
			
			//calculate outer most x and y points
			Rectangle canvasRect = new Rectangle(chart.CanvasLeft,chart.CanvasTop,chart.CanvasRight-chart.CanvasLeft,
				chart.CanvasBottom-chart.CanvasTop);
			
			Point[] points = calculateRectangleLineIntersection(canvasRect,x1,y1,x2,y2);
			if (points ==null) return;
			
			//draw ray or extended line, using c# Drawing library			
			g.SmoothingMode = SmoothingMode.AntiAlias;
			Pen pen = new Pen(new SolidBrush(color));
			pen.DashStyle = DashStyle.Dot;
			
			if (extended==true) {
				g.DrawLine(pen,points[0].X,points[0].Y,points[1].X,points[1].Y);
			} else {
				Point p;
				if (position==1 || position ==2) {
					//get right-most point
					if (points[0].X>points[1].X) 
						p = points[0];
					else
						p = points[1];
				} else {
					//get left-most point
					if (points[0].X<points[1].X) 
						p = points[0];
					else
						p = points[1];
				}			
				g.DrawLine(pen,(int)x1,(int)y1,p.X,p.Y);
			}
			pen.Dispose();		
			
		}
		
	}

public class FibLine: BaseFibLine
	{		

		public FibLine (FibSR indicator,FibonacciCircle circle, double level, int position, Color color,bool extended):base(indicator,circle,level,position,color,extended) {}
		
	
		public override void delete () {}


		
		public static double[] get_tangent_line(IndicatorBase indicator, FibonacciCircle circle, double ratio, int pos)  {
			Point start = getStart(circle,indicator);
			Point end = getEnd(circle,indicator);
			
			double x_start = start.X;
			double y_start = start.Y;
			double x_end = end.X;
			double y_end = end.Y;
				
			double deltax = x_end-x_start;
			double deltay = y_end-y_start;
			
			double radius = Math.Sqrt(deltax*deltax+deltay*deltay);
			double outer_radius = radius*1.618;
			radius = ratio/100.0*radius; 	
			
			//circle center coordinates
			double x = x_start;
			double y = -1*y_start;
			
			//top point coordinates
			double x1=0,y1=0;
			if (pos==1 || pos ==2) {
				x1 = x;
				y1 = (y+outer_radius);
			} else {
				x1 = x;
				y1 = (y - outer_radius);
			}
			
			//tangent point coordinates
			double x2 = -1*radius/(y1-y)*Math.Sqrt((y1-y)*(y1-y)-radius*radius);
			double y2 = (radius*radius/(y1-y)+y);
			if (pos==2 || pos==3) {
				x2 = -x2;
			}
			x2 = x2+ x;
			
			double[] pt_array = {x2,y2,x1,y1};
			return pt_array;
		}
		
		public override void draw (Graphics g, Rectangle bounds, double min, double max) 	
		{		
			
			double [] coordinates = FibLine.get_tangent_line(indicator,FibonacciCircle,this.level,this.position);
			double x2 = coordinates[0]; double y2 = coordinates[1];
			double x1 = coordinates[2]; double y1 = coordinates[3];

			// convert o drawing canvas y coordinates
			y1 = y1*-1;
			y2 = y2*-1;
			
			//calculate outer most x and y points
			Rectangle canvasRect = new Rectangle(chart.CanvasLeft,chart.CanvasTop,chart.CanvasRight-chart.CanvasLeft,
				chart.CanvasBottom-chart.CanvasTop);	
			Point[] points = calculateRectangleLineIntersection(canvasRect,x1,y1,x2,y2);
			if (points ==null) return;
			
			//draw lines
			g.SmoothingMode = SmoothingMode.AntiAlias;
			Pen pen = new Pen(new SolidBrush(color));
			pen.DashStyle = DashStyle.Dot;
			

			//draw ray or extended line, using c# Drawing library			
			
			if (extended==true) {
				g.DrawLine(pen,points[0].X,points[0].Y,points[1].X,points[1].Y);
			} else {
				Point p;
				if (position==1 || position ==2) {
					//get bottom point
					if (points[0].Y>points[1].Y) 
						p = points[0];
					else
						p = points[1];
				} else {
					//get top point
					if (points[0].Y<points[1].Y) 
						p = points[0];
					else
						p = points[1];
				}			
				g.DrawLine(pen,(int)x1,(int)y1,p.X,p.Y);
			}
			pen.Dispose();		
		}
	}
}

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public class FibSR : Indicator
    {
		//indicator  variables
		private string fibTag = "fib";
		private bool extended = true;
		private bool horizontal_line = true; 
		private bool vertical_lines  = false;
		private bool sr_lines = true;
		private bool offset_line = false;
		
		Color color1 = Color.LimeGreen;
		Color color2 = Color.NavajoWhite;	
		Color vertical_line_color = Color.DodgerBlue;
		Color horizontal_line_color = Color.LimeGreen;
		Color offset_line_color = Color.Orchid;
	    // 
		Pen pen;
		Pen vertical_line_pen;
		
	    private string mode = "price";
		bool new_position = true;
        int position = 1; 
		
		static double [] extensions = {38.2,61.8,100,138.2};
		FibLine [] lines = new FibLine[extensions.Length*2];
		FibTimeLine [] time_lines = new FibTimeLine[extensions.Length*2];

		protected override void Initialize() {
			Overlay	= true;
			CalculateOnBarClose = true;
			ChartOnly = true;
			pen =  new Pen(new SolidBrush(horizontal_line_color));
			vertical_line_pen = new Pen(new SolidBrush(vertical_line_color));
			vertical_line_pen.DashStyle = DashStyle.Dot;
			pen.DashStyle = DashStyle.Dot;
		}
			
		protected override void OnTermination () {
			delete();	//not needed, but just in case the code changes in the future .... 	
			pen.Dispose();
			vertical_line_pen.Dispose();
			if (buttonsloaded==true)
			{
				toolStrip.Items.Remove(btnManual);
				toolStrip.Items.Remove(sep1);
				toolStrip.Items.Remove(sep2);
				toolStrip.Items.Remove(fibPosition);
				if (OffsetLine==true) {
					toolStrip.Items.Remove(sep3);
					toolStrip.Items.Remove(offsetLine);
				}
			}
		}	
		
		//0 = left, 1 = up, 2 = right, 3 = down
		//set position if the first element in the array is null (has been deleted)
		public bool setPosition() {
			FibonacciCircle fib = getFibonacciCircle();
			if (fib==null) return false;
			if ((mode=="price" && lines[0]==null) || (mode=="time" && time_lines[0] ==null)) {
				for (int i=0;i<lines.Length/2;i++) {
					if (position==0) {
						time_lines[2*i] = new FibTimeLine(this,fib,extensions[i],1,Color1,extended);
						time_lines[2*i+1] = new FibTimeLine(this,fib,extensions[i],2,Color2,extended);
					} 
					else if (position==2) {	
						time_lines[2*i]  = new FibTimeLine(this,fib,extensions[i],3,Color1,extended);
						time_lines[2*i+1] = new FibTimeLine(this,fib,extensions[i],4,Color2,extended);
					} 
					else if (position==3) {	
						lines[2*i]  = new FibLine(this,fib,extensions[i],3,Color1,extended);
						lines[2*i+1] =  new FibLine(this,fib,extensions[i],4,Color2,extended);
					} 
					else if (position==1) {
						lines[2*i] = new FibLine(this,fib,extensions[i],1,Color1,extended);
						lines[2*i+1] = new FibLine(this,fib,extensions[i],2,Color2,extended);
					}					
				}
			}
			return true;
		}	
		
		//just leave the adjustment anchor with the start and end point on the chart!
		private void setFibonacciCircle (FibonacciCircle circle) {

			//no new murray circle found
			if (circle==null) return;
			//set everything
			FibonacciLevel [] newLevels = {new FibonacciLevel(), new FibonacciLevel(),new FibonacciLevel(), new FibonacciLevel(),
			new FibonacciLevel(), new FibonacciLevel(), new FibonacciLevel(), new FibonacciLevel(), new FibonacciLevel(), new FibonacciLevel()};
			
			circle.ShowText = false;
			circle.Levels = newLevels;	
		}
		
		public override void Plot(Graphics g, Rectangle bounds, double min, double max) {
			base.Plot(g, bounds, min, max);		
			delete();	
			
			FibonacciCircle fibCircle = getFibonacciCircle();
			if (fibCircle==null || fibCircle.DivideTimePrice==true) return;
			setFibonacciCircle(fibCircle);
			
			if (horizontal_line==true && getFibonacciCircle()!=null) {
				int [] y;
				y = BaseFibLine.get_horizontal_line(this,getFibonacciCircle(),true);	
				   g.DrawLine(pen,ChartControl.CanvasLeft,y[0],ChartControl.CanvasRight,y[0]);
					g.DrawLine(pen,ChartControl.CanvasLeft,y[1],ChartControl.CanvasRight,y[1]); 		    
			}
				
			if (vertical_lines==true && getFibonacciCircle()!=null) {
				FibonacciCircle circle = getFibonacciCircle();
				Point start = BaseFibLine.getStart(circle,this);
				Point end = BaseFibLine.getEnd(circle,this);
				double deltax = Math.Abs(start.X-end.X);
				double deltay = Math.Abs(start.Y-end.Y);
				double radius = Math.Sqrt(deltax*deltax+deltay*deltay);
				double [] level_radius = {radius*1.618};
				for (int i=0;i<level_radius.Length;i++) {
					int x1 = (int)(start.X-level_radius[i]);
					int x2 = (int)(start.X + level_radius[i]);
					g.DrawLine(vertical_line_pen,x1,ChartControl.Bottom,x1,ChartControl.Top);
					g.DrawLine(vertical_line_pen,x2,ChartControl.Bottom,x2,ChartControl.Top);
				}		
			}
			
			try {
				
			//draw offset line
			if (OffsetLine==true && getFibonacciCircle()!=null  && Extended==true) {
				string text = offsetLine.Text;
				string[] pos = text.Split((char[])null, StringSplitOptions.RemoveEmptyEntries);
				
				if (pos.Length==3) {
						double ratio = double.Parse(pos[0]);
						double radius_tangent = double.Parse(pos[1]);
						int radius_position = int.Parse(pos[2]);
						int ratio_position = radius_position;
					
						if ((radius_position>=1 && radius_position<=4)) 
						{
							
							Point start = BaseFibLine.getStart(getFibonacciCircle(),this);
							double center_x = start.X;
							double center_y = -start.Y;
							
							double ratio_multiple = radius_tangent/ratio;	
							double [] coords = null;
							if (mode=="price") 
								coords = FibLine.get_tangent_line(this,getFibonacciCircle(),ratio,ratio_position);
							else
								coords = FibTimeLine.get_tangent_line(this,getFibonacciCircle(),ratio,ratio_position);
							
							
							double x1 = (coords[0]-center_x)*ratio_multiple + center_x;
							double y1 = (coords[1]-center_y)*ratio_multiple + center_y;
							
							/*
							if ((ratio_position==1 && radius_position == 4) || (ratio_position ==4 && radius_position==1) ||
								(ratio_position ==2 && radius_position ==3) || (ratio_position == 3 && radius_position ==2 )) {
									x1 = -(x1 - center_x)+center_x;
									y1 = -(y1- center_y) + center_y;
							} 
							*/
								
						    if (mode=="time" && (radius_position == 2 || radius_position==3)) {
								x1 = -(x1 - center_x)+center_x;
									y1 = -(y1- center_y) + center_y;
							}
							
							//convert to drawing coordinates (negative y axis is positive)	
							y1 = -y1;
							double slope = -(coords[3]-coords[1])/(coords[2]-coords[0]); //slope in drawing coordinates (negative y)
									
								
							//calculate outer most x and y points
							Rectangle canvasRect = new Rectangle(ChartControl.CanvasLeft,ChartControl.CanvasTop,ChartControl.CanvasRight-ChartControl.CanvasLeft,
																 ChartControl.CanvasBottom-ChartControl.CanvasTop);	
							Point[] points = FibLine.calculateRectangleLineIntersection(canvasRect,x1,y1,slope);
								
							if (points !=null) {
								//draw lines
								g.SmoothingMode = SmoothingMode.AntiAlias;
								Pen pen = new Pen(new SolidBrush(offset_line_color));
								pen.DashStyle = DashStyle.Dot;
								g.DrawLine(pen,points[0].X,points[0].Y,points[1].X,points[1].Y);
							
								int width = 6;
								Rectangle rect = new Rectangle((int)(x1-width),(int)(y1-width),2*width,2*width);
								g.DrawEllipse(pen,rect);
								pen.Dispose();
							}
						}
					
					
				}
				
				
			}
			
			} catch (Exception e) {};
			
			if (sr_lines==true) 
			{
				if(!setPosition()) return;	
				for (int i=0;i<lines.Length;i++) {
					if (mode=="price")
						lines[i].draw(g,bounds,min,max);
					if (mode=="time")
						time_lines[i].draw(g,bounds,min,max);	
				}
			}	
		}
		
		protected override void OnBarUpdate() {}	
	
		private void delete() { 				
			FibonacciCircle fib = getFibonacciCircle();
			if (mode=="price") {
				if (time_lines[0]!=null) {
					for (int i=0;i<time_lines.Length;i++) {
						if (time_lines[i]!=null)
							time_lines[i].delete();
						time_lines[i]=null;
					}
				}
				if (new_position==true || (lines[0]!=null && lines[0].FibonacciCircle != fib)) {
					for (int i=0;i<lines.Length;i++) {
						if (lines[i]!=null)
							lines[i].delete();
						lines[i] = null;
					}
					new_position = false;
				}
			}
			else if (mode=="time") {
				if (lines[0]!=null) {
					for (int i=0;i<lines.Length;i++) {
						if (lines[i]!=null) 
							lines[i].delete();
						lines[i]=null;
					}
				}
				if (new_position==true || (time_lines[0]!=null && time_lines[0].FibonacciCircle!=fib)) {
					for (int i=0;i<time_lines.Length;i++) { 
						if (time_lines[i]!=null) 
							time_lines[i].delete();
						time_lines[i]=null;
					}
					new_position = false;
				}
			}				
		}
			
		private FibonacciCircle getFibonacciCircle() {
			IDrawObject obj = DrawObjects[FibTag];
			if (obj!=null && obj.DrawType == DrawType.FibonacciCircle) {
				return obj as FibonacciCircle;
			}
			return null;	
		}	
		
		#region Toolbar
		private System.Windows.Forms.ToolStripButton btnManual = null;
		private System.Windows.Forms.ToolStripButton fibPosition = null;
		
		private System.Windows.Forms.ToolStripSeparator sep1 = null;
		private System.Windows.Forms.ToolStripSeparator sep2 = null;
		private System.Windows.Forms.ToolStripSeparator sep3 = null;
        private System.Windows.Forms.ToolStrip toolStrip = null;
		
		private System.Windows.Forms.ToolStripTextBox offsetLine = null;
		
		
		private Font boldFont = new Font("Arial", 8,FontStyle.Bold);
		private Font regularFont = new Font("Arial", 8);
		private bool buttonsloaded = false;
			
		protected override void OnStartUp()
		{
			offsetLine = new System.Windows.Forms.ToolStripTextBox();
			offsetLine.AcceptsReturn = false;
			offsetLine.AcceptsTab = false;
			offsetLine.Multiline = false;
			offsetLine.Width = 100;
			offsetLine.HideSelection = false;

		    toolStrip = (ToolStrip)ChartControl.Controls["tsrTool"];
		    sep1 = new ToolStripSeparator();
			sep2 = new ToolStripSeparator();
			sep3 = new ToolStripSeparator();
			toolStrip.Items.Add(sep1);
			
			btnManual = new System.Windows.Forms.ToolStripButton("btnManual");
			fibPosition = new System.Windows.Forms.ToolStripButton("fibPosition");
			
			btnManual.Font = regularFont ;    
			btnManual.Font = boldFont ;
			btnManual.ForeColor = Color.Gainsboro;
			btnManual.BackColor= Color.Green;    
			btnManual.Text = mode;
			btnManual.ToolTipText = "Toggles FibSR type";

			fibPosition.Font = regularFont ;    
			fibPosition.Font = boldFont ;
			fibPosition.ForeColor = Color.Gainsboro;
			fibPosition.BackColor= Color.Green;    
			fibPosition.Text = "up";
			fibPosition.ToolTipText = "Toggles FibSR position";

			if (OffsetLine == true) {
				
				toolStrip.Items.Add(offsetLine);
				toolStrip.Items.Add(sep3);
			}
			
			toolStrip.Items.Add(btnManual);
			toolStrip.Items.Add(sep2);
			toolStrip.Items.Add(fibPosition);
	
			
			
			
			
			
			btnManual.Click += btnManual_Click;
			fibPosition.Click += fibPosition_Click;
			buttonsloaded=true;	
		}
		
		private void fibPosition_Click(object sender,EventArgs e) 
		{
			if (fibPosition.Text == "right") {
				fibPosition.Text = "left";
				position = 0;
			}
			else if (fibPosition.Text=="left") {
				fibPosition.Text = "right";
				position = 2;
			}
			else if (fibPosition.Text=="up") {
				fibPosition.Text = "down";
				position = 3;
			}
			else if (fibPosition.Text=="down") {
				fibPosition.Text = "up";
				position = 1;
			}
			new_position = true;
		}
		
		private void btnManual_Click(object sender, EventArgs e)
		{

	    if (btnManual.Text=="price") {
				btnManual.Text = "time";
			    mode="time";
			    fibPosition.Text = "left";
			    position =0; new_position = true;
				
		}
		else if (btnManual.Text =="time") {
			mode="price";
			btnManual.Text = "price";
			fibPosition.Text = "up"; position = 1; new_position = true;
		}
		return;		
		}
#endregion
			
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove	
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }
		
	
		
        [Description("true = lines, false = rays")]
        [GridCategory("Parameters")]
		public bool Extended 
		{
			get{return extended;}
			set{extended=value;}
		}	
		
		 [Description("true = horizontal line, false = no horizontal line")]
        [GridCategory("Parameters")]
		public bool HorizontalLine 
		{
			get{return horizontal_line;}
			set{horizontal_line=value;}
		}	
		
		 [Description("true = vertical line, false = no vertical line")]
        [GridCategory("Parameters")]
		public bool VerticalLines
		{
			get{return vertical_lines;}
			set{vertical_lines=value;}
		}
		
		 [Description("true = sr lines, false = no sr lines")]
        [GridCategory("Parameters")]
		public bool SRLines
		{
			get{return sr_lines;}
			set{sr_lines=value;}
		}
		
		
		[XmlIgnore()]
		[Description("SR Lines Color 1")]
        [GridCategory("Parameters")]
		public Color Color1 
		{
			get {return color1;}
			set {color1=value;}
		}
		
		[Browsable(false)]
		public string Color1Serialize
		{
     		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(color1); }
     		set { color1 = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		[XmlIgnore()]
		[Description("SR Lines Color 2")]
        [GridCategory("Parameters")]
		public Color Color2
		{
			get{return color2;}
			set {color2=value;}
		}
		
		[Browsable(false)]
		public string Color2Serialize
		{
     		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(color2); }
     		set { color2 = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		} 
		
		 [Description("true = offset sr line, false = no offset line")]
        [GridCategory("Parameters")]
		public bool OffsetLine
		{
			get{return offset_line;}
			set{offset_line=value;}
		}
			[XmlIgnore()]
		[Description("Offset Line Color")]
        [GridCategory("Parameters")]
		public Color OffsetLineColor 
		{
			get {return offset_line_color;}
			set {offset_line_color=value;}
		}
		
		
		[Description("tag of fibonacci circle")]
        [GridCategory("Parameters")]	
		public string FibTag
		{
			get {return fibTag;}
			set {fibTag = value;}
		}
		
		
		
		[XmlIgnore()]
		[Description("Horizontal Line Color")]
        [GridCategory("Parameters")]
		public Color HorizontalLineColor
		{
			get{return horizontal_line_color;}
			set {horizontal_line_color=value;}
		}
		
		[Browsable(false)]
		public string HorizontalLineColorSerializable
		{
     		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(horizontal_line_color); }
     		set { horizontal_line_color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		} 
		
		[XmlIgnore()]
		[Description("Vertical Line Color")]
        [GridCategory("Parameters")]
		public Color VerticalLineColor
		{
			get{return vertical_line_color;}
			set {vertical_line_color=value;}
		}
		
		[Browsable(false)]
		public string VerticalLineColorSerializable
		{
     		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(vertical_line_color); }
     		set { vertical_line_color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
	
        #endregion
    }
}


#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private FibSR[] cacheFibSR = null;

        private static FibSR checkFibSR = new FibSR();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public FibSR FibSR(Color color1, Color color2, bool extended, string fibTag)
        {
            return FibSR(Input, color1, color2, extended, fibTag);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public FibSR FibSR(Data.IDataSeries input, Color color1, Color color2, bool extended, string fibTag)
        {
            if (cacheFibSR != null)
                for (int idx = 0; idx < cacheFibSR.Length; idx++)
                    if (cacheFibSR[idx].Color1 == color1 && cacheFibSR[idx].Color2 == color2 && cacheFibSR[idx].Extended == extended && cacheFibSR[idx].FibTag == fibTag && cacheFibSR[idx].EqualsInput(input))
                        return cacheFibSR[idx];

            lock (checkFibSR)
            {
                checkFibSR.Color1 = color1;
                color1 = checkFibSR.Color1;
                checkFibSR.Color2 = color2;
                color2 = checkFibSR.Color2;
                checkFibSR.Extended = extended;
                extended = checkFibSR.Extended;
                checkFibSR.FibTag = fibTag;
                fibTag = checkFibSR.FibTag;

                if (cacheFibSR != null)
                    for (int idx = 0; idx < cacheFibSR.Length; idx++)
                        if (cacheFibSR[idx].Color1 == color1 && cacheFibSR[idx].Color2 == color2 && cacheFibSR[idx].Extended == extended && cacheFibSR[idx].FibTag == fibTag && cacheFibSR[idx].EqualsInput(input))
                            return cacheFibSR[idx];

                FibSR indicator = new FibSR();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Color1 = color1;
                indicator.Color2 = color2;
                indicator.Extended = extended;
                indicator.FibTag = fibTag;
                Indicators.Add(indicator);
                indicator.SetUp();

                FibSR[] tmp = new FibSR[cacheFibSR == null ? 1 : cacheFibSR.Length + 1];
                if (cacheFibSR != null)
                    cacheFibSR.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheFibSR = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FibSR FibSR(Color color1, Color color2, bool extended, string fibTag)
        {
            return _indicator.FibSR(Input, color1, color2, extended, fibTag);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.FibSR FibSR(Data.IDataSeries input, Color color1, Color color2, bool extended, string fibTag)
        {
            return _indicator.FibSR(input, color1, color2, extended, fibTag);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FibSR FibSR(Color color1, Color color2, bool extended, string fibTag)
        {
            return _indicator.FibSR(Input, color1, color2, extended, fibTag);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.FibSR FibSR(Data.IDataSeries input, Color color1, Color color2, bool extended, string fibTag)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.FibSR(input, color1, color2, extended, fibTag);
        }
    }
}
#endregion
